;*******************************************************************************
;*
;*      Title:  Subroutine to evaluate and execute Commands from EIA232 or USB
;*              (SR_COMMANDS.asm)
;*
;*      Date:   31/01/14        Version:        0.1
;*      Author:  WEIGU
;*
;*      Hardware:
;*      Controller:     ATmega32A/644p/32u4     Crystal:  
;*      Input:
;*      Output:
;*
;*      Fundamentals of operation:
;*      
;*      Copyright (c) 2014   Guy WEILER         weigu[at]weigu[dot]lu
;*
;*      more info on www.weigu.lu/a and www.weigu.lu/b
;*      
;*      This program is free software: you can redistribute it and/or modify
;*      it under the terms of the GNU General Public License as published by
;*      the Free Software Foundation, either version 3 of the License, or
;*      (at your option) any later version.
;*
;*      This program is distributed in the hope that it will be useful,
;*      but WITHOUT ANY WARRANTY; without even the implied warranty of
;*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;*      GNU General Public License for more details.
;*
;*      You should have received a copy of the GNU General Public License
;*      along with this program.  If not, see <http://www.gnu.org/licenses/>.
;*
;*      Licence: GPL-3
;*
;*******************************************************************************

; Please select only the subroutines needed (Flag = 1). This helps to save Flash 
; memory
.EQU    Flag_COMAN    = 1       ;analyse the command and call subroutine
.EQU    Flag_COM_L    = 1       ;send command list
.EQU    Flag_COM_G    = 1       ;send minute data 
.EQU    Flag_COM_Q    = 1       ;send quarter data 
.EQU    Flag_COM_D    = 1       ;send day data 
.EQU    Flag_COM_A    = 1       ;get all data (quarter 'aq' or day 'ad')
.EQU    Flag_COM_P    = 1       ;get the 2 pointer (q and d) 
.EQU    Flag_COM_S    = 1       ;set the 2 pointer (q and d) 
.EQU    Flag_COM_C    = 1       ;clear EEPROM and reset the 2 pointer
.EQU    Flag_COM_T    = 1       ;set the RTC with the data from EIA232/USB
.EQU    Flag_SNDPGE   = 1       ;send page data (needed by COM_Q + COM_D)
.EQU    Flag_COMOK    = 1       ;send "OK"
.EQU    Flag_COMER    = 1       ;send "SYNTAX ERROR"

;///////////////////////////////////////////////////////////////////////////////
;       Makros 
;       from http://www.mikrocontroller.net/articles/AVR_Assembler_Makros
;///////////////////////////////////////////////////////////////////////////////
; The newer controller are using more than 64 SF-register (Special Function reg.
; or I/O reg.). These higher register can't be accessed with the commands "in"
; and "out" but with "lds" and "sts". In this library the macros "input" and
; "output" are used to access all SF-Register.

;Read an SF-register with "input working-register, SF-register"
.IFNDEF MACROINPUT
.SET    MACROINPUT = 1
  .MACRO input
    .IF @1 < 64
        in      @0, @1
    .ELSE
        lds     @0, @1
    .ENDIF
  .ENDM
.ENDIF

;Write to an SF-register with "output SF-register,working-register"
.IFNDEF MACROOUTPUT
.SET    MACROOUTPUT = 1
  .MACRO output
    .IF @0 < 64
        out     @0, @1
    .ELSE
        sts     @0, @1
    .ENDIF
  .ENDM
.ENDIF

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Reserving bytes and assignments
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;.DSEG
;.CSEG

;-------------------------------------------------------------------------------
;       Analysing the command and call the corresponding subroutine
;       (the set time 't' command is treated in the EIA232 or USB routine)
;-------------------------------------------------------------------------------
.IF Flag_COMAN == 1
COMAN:  push    r16             
        push    r17    
        push    XL    
        push    XH    

        lds     r16,RX232DATA   ;get the first character
        cpi     r16,'?'         ;analyse commands and call corresponding 
        brne    COMAN1          ;subroutine
        rcall   COM_L
        rjmp    COMANR
COMAN1: cpi     r16,'g'         
        brne    COMAN2
        rcall   COM_G
        rjmp    COMANR
COMAN2: cpi     r16,'q' 
        brne    COMAN3
        rcall   COM_Q
        rjmp    COMANR
COMAN3: cpi     r16,'d' 
        brne    COMAN4
        rcall   COM_D
        rjmp    COMANR
COMAN4: cpi     r16,'a' 
        brne    COMAN5
        rcall   COM_A
        rjmp    COMANR
COMAN5: cpi     r16,'p' 
        brne    COMAN6
        rcall   COM_P
        rjmp    COMANR
COMAN6: cpi     r16,'s' 
        brne    COMAN7
        rcall   COM_S
        rjmp    COMANR
COMAN7: cpi     r16,'c' 
        brne    COMAN8
        rcall   COM_C
        rjmp    COMANR
COMAN8:                         ;continue here if more commands!

COMANR: pop     XH    
        pop     XL    
        pop     r17
        pop     r16             
        ret
.ENDIF    

;-------------------------------------------------------------------------------
;       write the command list         
;-------------------------------------------------------------------------------
.IF Flag_COM_L == 1
COM_L:  push    r16
        
        ldi     r16,HIGH(COMLST*2)      ;initialize pointer
        sts     TX232P+0,r16
        ldi     r16,LOW(COMLST*2)       
        sts     TX232P+1,r16
        input   r16,UCSRB               ;enable UDRIE
        ori     r16,0x20         
        output  UCSRB,r16    

        pop     r16
        ret
.ENDIF    

;-------------------------------------------------------------------------------
;       send minute data 
;       date + time + 8 data bytes (hex) + CR (41 bytes)
;-------------------------------------------------------------------------------
.IF Flag_COM_G == 1
COM_G:  push    r18
        push    r24
        push    XL
        push    XH

        rcall   RTCEIA 
        ldi     r18,8
        ldi     XL,LOW(DBUFFM)
        ldi     XH,HIGH(DBUFFM)        
COM_GL: ld      r24,X+          
        rcall   E232HEX 
        dec     r18
        brne    COM_GL
        ldi     r24,0x0D
        rcall   E232SND
        rcall   COMOK

        pop     XH
        pop     XL
        pop     r24
        pop     r18             
        ret
.ENDIF    

;-------------------------------------------------------------------------------
;       send quarter data 
;       dd.mm.yy+Space+hh.MM+Space+Space (16 byte) + 7 dataset * 16 byte
;-------------------------------------------------------------------------------
.IF Flag_COM_Q == 1
COM_Q:  push    r24
        push    r25
        
        ldi     r25,0x00        ;get current address (result in EEPDAT)
        ldi     r24,0x00        ;HByte
        rcall   EEPRRD
        lds     r24,EEPDAT
        sts     E24ADH,r24
        ldi     r24,0x01        ;LByte
        rcall   EEPRRD
        lds     r24,EEPDAT
        sts     E24ADL,r24
        rcall   SNDPGE          
        rcall   COMOK

        pop     r25
        pop     r24
        ret
.ENDIF    

;-------------------------------------------------------------------------------
;       send day data 
;       dd.mm.yy+Space+hh.MM+Space+Space (16 byte) + 7 dataset * 16 byte
;-------------------------------------------------------------------------------
.IF Flag_COM_D == 1
COM_D:  push    r24
        push    r25
        
        ldi     r25,0x00        ;get current address (result in EEPDAT)
        ldi     r24,0x02        ;HByte
        rcall   EEPRRD
        lds     r24,EEPDAT
        sts     E24ADH,r24
        ldi     r24,0x03        ;LByte
        rcall   EEPRRD
        lds     r24,EEPDAT
        sts     E24ADL,r24
        rcall   SNDPGE          
        rcall   COMOK

        pop     r25
        pop     r24
        ret
.ENDIF        

;-------------------------------------------------------------------------------
;       get all data (quarter 'aq' or day 'ad')
;       Y as page counter, Z as loop counter
;-------------------------------------------------------------------------------
.IF Flag_COM_A == 1
COM_A:  push    r24
        push    r25
        push    YL
        push    YH
        push    ZL
        push    ZH

        lds     r24,RX232DATA+2         ;last char = CR?
        cpi     r24,0x0D                ;error control
        brne    COM_AE1
        lds     r24,RX232DATA+1         ;get the second char
        cpi     r24,'q'
        breq    COM_AQ
        cpi     r24,'d'
        breq    COM_AD
COM_AE1: rjmp   COM_AE                  ;error

COM_AQ: ;get all quarter data
        ldi     r25,0x00                ;get current address (result in EEPDAT)
        ldi     r24,0x00                ;HByte
        rcall   EEPRRD
        lds     YH,EEPDAT               ;save to page pointerH (YH)
        ldi     r24,0x01                ;LByte
        rcall   EEPRRD
        lds     YL,EEPDAT               ;save to page pointerL (YL)
COM_AQ1: ldi    ZL,LOW(448)             ;loop counter = 448 (pages)    
        ldi     ZH,HIGH(448)
COM_AQL: sts    E24ADH,YH               ;read EEPROM page (128byte)
        sts     E24ADL,YL               ;result in SRAM 
        rcall   SNDPGE                  ;send page
        sbiw    ZL,1
        breq    COM_AR1                 ;if loop counter = 0 return
        ldi     r24,128                 ;next address
        sub     YL,r24                  ;backwards
        sbc     YH,Zero
        cpi     YL,0x80                 ;if ringbuffer = 0xFF80 set to 0xDF80   
        brne    COM_AQL  
        cpi     YH,0xFF     
        brne    COM_AQL  
        ldi     YL,0x80
        ldi     YH,0xDF
        rjmp    COM_AQL 

COM_AD: ;get all day data
        ldi     r25,0x00                ;get current address (result in EEPDAT)
        ldi     r24,0x02                ;HByte
        rcall   EEPRRD
        lds     YH,EEPDAT               ;save to page pointerH (YH)
        ldi     r24,0x03                ;LByte
        rcall   EEPRRD
        lds     YL,EEPDAT               ;save to page pointerL (YL)
COM_AD1: ldi    ZL,64                   ;loop counter = 64 (pages)    
COM_ADL: sts    E24ADH,YH               ;read EEPROM page (128byte)
        sts     E24ADL,YL               ;result in SRAM 
        rcall   SNDPGE                  ;send page
        dec     ZL
        breq    COM_AR1                 ;if loop counter = 0 return
        ldi     r24,128                 ;next address
        sub     YL,r24                  ;backwards
        sbc     YH,Zero
        cpi     YL,0x80                 ;if ringbuffer = 0xDF80 set to 0xFFFF   
        brne    COM_ADL  
        cpi     YH,0xDF                 
        brne    COM_ADL  
        ldi     YL,0x80                 
        ldi     YH,0xFF                 
        rjmp    COM_ADL 

COM_AE: rcall   COMER                   ;error
        rjmp    COM_AR
COM_AR1: rcall  COMOK
COM_AR: pop     ZH
        pop     ZL
        pop     YH
        pop     YL
        pop     r25
        pop     r24
        ret
.ENDIF    

;-------------------------------------------------------------------------------
;       get the 2 pointer (q and d) and send them to EIA232
;-------------------------------------------------------------------------------
.IF Flag_COM_P == 1
COM_P:  push    r24   

        ldi     r25,0x00        ;get current q pointer (result in EEPDAT)
        ldi     r24,0x00        ;HByte
        rcall   EEPRRD        
        lds     r24,EEPDAT
        rcall   E232HEX         ;send HByte
        ldi     r24,0x01        ;LByte
        rcall   EEPRRD
        lds     r24,EEPDAT
        rcall   E232HEX         ;send LByte
        ldi     r24,0x02        ;get current d pointer (result in EEPDAT)
        rcall   EEPRRD
        lds     r24,EEPDAT
        rcall   E232HEX         ;send HByte
        ldi     r24,0x03
        rcall   EEPRRD
        lds     r24,EEPDAT
        rcall   E232HEX         ;send LByte
        ldi     r24,0x0D
        rcall   E232SND 
        rcall   COMOK

        pop     r24
        ret
.ENDIF    

;-------------------------------------------------------------------------------
;       set the 2 pointer (q and d) 
;       ex.: "s 0000E0000"
;-------------------------------------------------------------------------------
.IF Flag_COM_S == 1
COM_S:  push    r24   
        push    r25
        
        lds     r24,RX232DATA+1         ;get the second char
        cpi     r24,' '                 ;error control
        brne    COM_SE
        lds     r24,RX232DATA+10        ;last char = CR?
        cpi     r24,0x0D                ;error control
        brne    COM_SE
        lds     r25,RX232DATA+2
        lds     r24,RX232DATA+3
        rcall   ASCH2D2
        sts     EEPDAT,r24
        ldi     r25,0x00
        ldi     r24,0x00
        rcall   EEPRWR
        lds     r25,RX232DATA+4
        lds     r24,RX232DATA+5
        rcall   ASCH2D2
        sts     EEPDAT,r24
        ldi     r25,0x00
        ldi     r24,0x01
        rcall   EEPRWR
        lds     r25,RX232DATA+6
        lds     r24,RX232DATA+7
        rcall   ASCH2D2
        sts     EEPDAT,r24
        ldi     r25,0x00
        ldi     r24,0x02
        rcall   EEPRWR
        lds     r25,RX232DATA+8
        lds     r24,RX232DATA+9
        rcall   ASCH2D2
        sts     EEPDAT,r24
        ldi     r25,0x00
        ldi     r24,0x03
        rcall   EEPRWR
        rcall   COMOK
        rjmp    COM_SR
COM_SE: rcall   COMER
COM_SR: pop     r25
        pop     r24    
        ret
.ENDIF    

;-------------------------------------------------------------------------------
;       clear EEPROM and reset the 2 pointer
;-------------------------------------------------------------------------------
.IF Flag_COM_C == 1
COM_C:  push    r16
        push    r18
        push    r24
        push    r25
        push    YL
        push    YH
        
        ldi     r24,'P'         
        rcall   E232SND 
        ldi     r24,'l'
        rcall   E232SND 
        ldi     r24,'e'         
        rcall   E232SND 
        ldi     r24,'a'
        rcall   E232SND 
        ldi     r24,'s'         
        rcall   E232SND 
        ldi     r24,'e'
        rcall   E232SND 
        ldi     r24,' '         
        rcall   E232SND 
        ldi     r24,'w'
        rcall   E232SND 
        ldi     r24,'a'         
        rcall   E232SND 
        ldi     r24,'i'
        rcall   E232SND 
        ldi     r24,'t'         
        rcall   E232SND 
        
        sts     E24ADH,Zero             ;EEPROM address = 0x0000
        sts     E24ADL,Zero        
        
        ldi     YL,LOW(E24PAGE)         ;clear SRAM table (128 byte)
        ldi     YH,HIGH(E24PAGE)        
        ldi     r18,128
CERPL1: st      Y+,Zero
        dec     r18
        brne    CERPL1
        
CERPL2: ldi     YL,LOW(E24PAGE)         ;save EEPROM adress to first 16 Bytes
        ldi     YH,HIGH(E24PAGE)        ;of SRAM table
        ldi     r16,'0'
        st      Y+,r16
        ldi     r16,'x'
        st      Y+,r16
        lds     r24,E24ADH              
        lds     r16,E24ADL              
        rcall   HEX2A2
        st      Y+,r25
        st      Y+,r24
        mov     r24,r16
        rcall   HEX2A2
        st      Y+,r25
        st      Y+,r24
        ldi     r16,' '
        ldi     r18,10
CERPL3: st      Y+,r16
        dec     r18
        brne    CERPL3
        
        call    E24SP                   ;table to eeprom
        ldi     r24,'.'                         
        rcall   E232SND 
        
        ldi     r16,128                 ;inc EEPROM address (128 byte)
        lds     r25,E24ADH
        lds     r24,E24ADL              
        add     r24,r16
        adc     r25,Zero
        sts     E24ADH,r25
        sts     E24ADL,r24
        cpi     r24,0                   ;til address = 0x0000
        brne    CERPL2
        cpi     r25,0
        brne    CERPL2

        sts     EEPDAT,Zero             ;q pointer = 0x0000
        ldi     r25,0x00
        ldi     r24,0x00
        rcall   EEPRWR
        ldi     r24,0x01
        rcall   EEPRWR
        ldi     r24,0x03
        rcall   EEPRWR
        ldi     r16,0xE0                ;d Pointer = 0xE000
        sts     EEPDAT,r16
        ldi     r24,0x02
        rcall   EEPRWR
        rcall   W10ms                   ;wait to be sure the saving ends
        ldi     r24,0x0D
        rcall   E232SND 
        rcall   COMOK

        pop     YH        
        pop     YL
        pop     r25
        pop     r24
        pop     r18
        pop     r16
        ret        
.ENDIF    

;-------------------------------------------------------------------------------
;       Command to set the RTC with the data from EIA232/USB
;-------------------------------------------------------------------------------
.IF Flag_COM_T == 1
COM_T:  push    r16             
        push    r17  

        lds     r16,RX232DATA+1         ;get the second char
        cpi     r16,' '                 ;error control
        brne    COM_TE
        lds     r16,RX232DATA+12        ;last char = CR?
        cpi     r16,0x0D                ;error control
        brne    COM_TE
        lds     r16,RX232DATA+2         ;day
        subi    r16,0x30
        swap    r16
        lds     r17,RX232DATA+3
        subi    r17,0x30
        add     r16,r17
        sts     RTCTAB+4,r16
        lds     r16,RX232DATA+4         ;month
        subi    r16,0x30
        swap    r16
        lds     r17,RX232DATA+5
        subi    r17,0x30
        add     r16,r17
        sts     RTCTAB+5,r16
        lds     r16,RX232DATA+6         ;year
        subi    r16,0x30
        swap    r16
        lds     r17,RX232DATA+7
        subi    r17,0x30
        add     r16,r17
        sts     RTCTAB+6,r16
        lds     r16,RX232DATA+8         ;hour
        subi    r16,0x30
        swap    r16
        lds     r17,RX232DATA+9
        subi    r17,0x30
        add     r16,r17
        sts     RTCTAB+2,r16
        lds     r16,RX232DATA+10        ;minutes
        subi    r16,0x30
        swap    r16
        lds     r17,RX232DATA+11
        subi    r17,0x30
        add     r16,r17
        sts     RTCTAB+1,r16
        clr     r16                     ;seconds
        sts     RTCTAB+0,r16
        ldi     Tmp1,0x00               ;control byte SQW off
        sts     RTCTAB+7,r16 

        rcall   RTCSND                  ;set RTC
        rcall   COMOK
        rjmp    COM_TR
        
COM_TE: rcall   COMER        
COM_TR: pop     r17
        pop     r16    
        ret
.ENDIF    

;-------------------------------------------------------------------------------
;       send page data  (128 byte)
;       adress in E24ADH and E24ADL
;       dd.mm.yy+Space+hh.MM+Space+Space (16 byte) + 7 dataset * 16 byte
;       (needed by COM_Q + COM_D)
;-------------------------------------------------------------------------------
.IF Flag_SNDPGE == 1
SNDPGE: push    r18
        push    r24
        push    XL
        push    XH
        
        rcall   E24RS           ;fetch data
        ldi     r18,16          ;date and time
        ldi     XL,LOW(E24PAGE) 
        ldi     XH,HIGH(E24PAGE)
SNDPG1: ld      r24,X+
        rcall   E232SND
        dec     r18
        brne    SNDPG1
        ldi     r18,112         ;2*16 databyte
        ldi     XL,LOW(E24PAGE+16) 
        ldi     XH,HIGH(E24PAGE+16)         
SNDPG2: ld      r24,X+          
        rcall   E232HEX 
        dec     r18
        brne    SNDPG2
        ldi     r24,0x0D
        rcall   E232SND 

        pop     XH
        pop     XL
        pop     r24
        pop     r18             
        ret        
.ENDIF    

;-------------------------------------------------------------------------------
;       write "ok" to EIA232/USB
;-------------------------------------------------------------------------------
.IF Flag_COMOK == 1
COMOK:  push    r24

        ldi     r24,'O'         ;Bestaetigung
        rcall   E232SND 
        ldi     r24,'K'
        rcall   E232SND 
        ldi     r24,0x0D
        rcall   E232SND 
        
        pop     r24
        ret
.ENDIF    

;-------------------------------------------------------------------------------
;       write error to EIA232/USB
;-------------------------------------------------------------------------------
.IF Flag_COMER == 1
COMER:  push    r16

        ldi     r16,HIGH(COMERT*2)      ;initialize pointer
        sts     TX232P+0,r16
        ldi     r16,LOW(COMERT*2)       
        sts     TX232P+1,r16
        input   r16,UCSRB               ;enable UDRIE
        ori     r16,0x20         
        output  UCSRB,r16    

        
        pop     r16
        ret
.ENDIF    


COMLST: .DB     "command list:",0x0D    ;!! even number of bytes per line !!
        .DB     "the commands must terminate with a carriage return (0x0D)!"
        .DB     0x0D,0x0D
        .DB     "?  help",0x0D
        .DB     "g  get data (1 set)",0x0D
        .DB     "q  get quarter data (last page)",0x0D
        .DB     "d  get day data (last page)",0x0D
        .DB     "aq get all quarter data (448 pages)",0x0D
        .DB     "ad get all day data (64 pages) ",0x0D
        .DB     "p  get the 2 EEPROM Pointer (quarter + day)",0x0D
        .DB     "t  set RTC ",0x0D
        .DB     "   format:  t+space+date(ddmmjj)+time(hhmm)",0x0D
        .DB     "   example: t 2204121522 ",0x0D
        .DB     "s  set the 2 EEPROM Pointer (LByte = 0x00 or 0x80) ",0x0D
        .DB     "   format:  s+space+2byte(1pointer)+2byte(2pointer)",0x0D
        .DB     "   example: s 0000E000 ",0x0D
        .DB     "c  clear all and reset pointer",0x0D,0
        
        ;!! even number of bytes per line !!
COMERT: .DB    "SYNTAX ERROR! (see command list)",0x0D,0

