;*******************************************************************************
;*
;*      Titel:  Soft-UART-Bibliothek: Unterprogramme zur Emulation einer
;*              seriellen Schnittstelle (EIA-232)
;*              (SR_UART_SOFT.asm)
;*
;*      Datum:  15/01/11        Version:        1.0
;*      Autor:  WEIGU
;*
;*      Informationen zur Beschaltung:
;*
;*      Prozessor: ATmega32A       Quarzfrequenz: 16MHz
;*      Eingaenge: Festlegung per Zuweisung
;*      Ausgaenge: Festlegung per Zuweisung
;*
;*      Unterprogramme:
;*      SSNDC:     Sende 1 Byte (r24) ueber die serielle Schnittstelle (8N1)
;*      SSNDC11:   Sende 1 Byte (r24) ueber die serielle Schnittstelle
;*                 (115200 Baud 8N1) Quarz 16MHz!! Fehler unter 0,5%
;*      SRCVC:     Empfange 1 Byte (r24) ueber die serielle Schnittstelle (8N1)
;*      ISRSRC:    Empfange 1 Byte (r24) ueber die serielle Schnittstelle (8N1)
;*                 ISR nutzt externen Interrupt an PB2 (INT2)
;*
;*
;*      Copyright (c) 2011   Guy WEILER         weigu[at]weigu[dot]lu
;*
;*      Hiermit wird unentgeltlich, jeder Person, die eine Kopie der Software
;*      und der zugehoerigen Dokumentationen (die "Software") erhaelt, die
;*      Erlaubnis erteilt, uneingeschraenkt zu benutzen, inklusive und ohne
;*      Ausnahme, dem Recht, sie zu verwenden, kopieren, aendern, fusionieren,
;*      verlegen, verbreiten, unterlizenzieren und/oder zu verkaufen, und
;*      Personen, die diese Software erhalten, diese Rechte zu geben, unter den
;*      folgenden Bedingungen:
;*
;*      ;Der obige Urheberrechtsvermerk und dieser Erlaubnisvermerk sind in alle
;*      ;Kopien oder Teilkopien der Software beizulegen.
;*      ;
;*      ;DIE SOFTWARE WIRD OHNE JEDE AUSDRUECKLICHE ODER IMPLIZIERTE GARANTIE
;*      ;BEREITGESTELLT, EINSCHLIESSLICH DER GARANTIE ZUR BENUTZUNG FUER DEN
;*      ;VORGESEHENEN ODER EINEM BESTIMMTEN ZWECK SOWIE JEGLICHER RECHTS-
;*      ;VERLETZUNG, JEDOCH NICHT DARAUF BESCHRAENKT. IN KEINEM FALL SIND DIE
;*      ;AUTOREN ODER COPYRIGHTINHABER FUER JEGLICHEN SCHADEN ODER SONSTIGE
;*      ;ANSPRUECHE HAFTBAR ZU MACHEN, OB INFOLGE DER ERFUELLUNG EINES
;*      ;VERTRAGES, EINES DELIKTES ODER ANDERS IM ZUSAMMENHANG MIT DER SOFTWARE
;*      ;ODER SONSTIGER VERWENDUNG DER SOFTWARE ENTSTANDEN.
;*
;*******************************************************************************
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Zuweisungen 
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.EQU    RXDDR   = DDRC          ;DDR-Register fuers Empfangen
.EQU    RXPIN   = PINC          ;PIN-Register fuers Empfangen
.EQU    RXPORT  = PORTC         ;PORT-Register fuers Empfangen
.EQU    RXPNr   = 0             ;Pin Nummer fuers Empfangen

.EQU    TXDDR   = DDRA          ;DDR-Register fuers Senden
.EQU    TXPORT  = PORTA         ;Port-Register fuers Senden
.EQU    TXPNr   = 0             ;Pin Nummer fuers Senden

.EQU    Baud    = 19200         ;Baudrate
.EQU    Clock   = 16000000      ;Systemtakt (CPU)

;-------------------------------------------------------------------------------
;       Sende 1 Byte (r24) ueber die serielle Schnittstelle (8N1)
;-------------------------------------------------------------------------------
SSNDC:  push    XL              ;rette verwendete Register
        push    XH

        sbi     TXDDR,TXPNr     ;Ausgang
        sbi     TXPORT,TXPNr    ;Leitung auf 1
        ldi     r25,8           ;Zaehler = 8

        cbi     TXPORT,TXPNr    ;Startbit senden
        ;Zeitschleife fuer das Startbit
        ldi     XL,LOW(((((10*Clock)/(4*Baud))-((2*10)/4)+5)/10))
        ldi     XH,HIGH(((((10*Clock)/(4*Baud))-((2*10)/4)+5)/10))
SSNDC1: sbiw    XL,1
        brne    SSNDC1
        ;Sende Zeichen LSB first!
SSNDC2: lsr     r24             ;Bit in den Uebertrag (Carry) schieben
        brcs    SSNDC3          ;Carry-Bit testen
        cbi     TXPORT,TXPNr    ;Null
        rjmp    SSNDC4          ;zur Zeitschleife
SSNDC3: sbi     TXPORT,TXPNr    ;Eins
        nop                     ;Zeitausgleich
SSNDC4: ;Zeitschleife fuer ein Datenbit
        ldi     XL,LOW(((((10*Clock)/(4*Baud))-((9*10)/4)+5)/10))
        ldi     XH,HIGH(((((10*Clock)/(4*Baud))-((9*10)/4)+5)/10))
SSNDC5: sbiw    XL,1
        brne    SSNDC5
        dec     r25             ;falls nicht alle Bits durch, dann naechstes Bit
        brne    SSNDC2

        nop                     ;Zeitausgleich für Sprungbefehl (MSB)
        sbi     TXPORT,TXPNr    ;Stoppbit senden
        ;Zeitschleife fuer das Stoppbit
        ldi     XL,LOW(((((10*Clock)/(4*Baud))-((2*10)/4)+5)/10))
        ldi     XH,HIGH(((((10*Clock)/(4*Baud))-((2*10)/4)+5)/10))
SSNDC6: sbiw    XL,1
        brne    SSNDC6

        pop     XH              ;Register wiederherstellen
        pop     XL
        ret                     ;zurueck

;-------------------------------------------------------------------------------
;       Sende 1 Byte (r24) ueber die serielle Schnittstelle (115200 Baud 8N1)
;       Quarz 16MHz!! Fehler unter 0,5%
;-------------------------------------------------------------------------------
SSNDC11: push    r16

        sbi     TXDDR,TXPNr     ;Ausgang
        sbi     TXPORT,TXPNr    ;Leitung auf 1
        ldi     r25,8           ;Zaehler = 8

        cbi     TXPORT,TXPNr    ;Startbit
        ;Zeitschleife
        ldi     r16,46          ;115200 Baud = 139 Taktzyklen/Bit bei 16MHz
SSNDCA: dec     r16             ;8 Bit Zeitschleife t = 3GtT (138/3 = 46)
        brne    SSNDCA

SSNDCB: lsr     r24             ;Sende Zeichen LSB first!
        brcs    SSNDCC
        cbi     TXPORT,TXPNr    ;Null
        rjmp    SSNDCD
SSNDCC: sbi     TXPORT,TXPNr    ;Eins
        nop                     ;Zeitausgleich
SSNDCD: ;Zeitschleife
        ldi     r16,43          ;115200 Baud = 139 Taktzyklen/Bit bei 16MHz
SSNDCE: dec     r16             ;8 Bit Zeitschleife t = 3GtT (129/3 = 43)
        brne    SSNDCE
        nop                     ;Zeitausgleich fuer 2 Taktzyklen
        nop
        dec     r25
        brne    SSNDCB
        nop                     ;Zeitausgleich
        sbi     TXPORT,TXPNr    ;Eins
        ;Zeitschleife
        ldi     r16,46          ;115200 Baud = 139 Taktzyklen/Bit bei 16MHz
SSNDCF: dec     r16             ;8 Bit Zeitschleife t = 3GtT (138/3 = 46)
        brne    SSNDCF

        pop     r16
        ret

;-------------------------------------------------------------------------------
;       Empfange 1 Byte (r24) ueber die serielle Schnittstelle (8N1)
;-------------------------------------------------------------------------------
SRCVC:  cbi     RXDDR,RXPNr     ;Eingang
        sbi     RXPORT,RXPNr    ;Pull-Up

        push    XL              ;rette verwendete Register
        push    XH
        ldi     r25,8           ;Zaehler = 8

SRCVC1: sbic    RXPIN,RXPNr     ;Warte auf das Startbit
        rjmp    SRCVC1
        ;Zeitschleife halbe Bitzeit
        ldi     XL,LOW(((((10*Clock)/(4*Baud))-((1*10)/4)+5)/20))
        ldi     XH,HIGH(((((10*Clock)/(4*Baud))-((1*10)/4)+5)/20))
SRCVC2: sbiw    XL,1
        brne    SRCVC2
        sbic    RXPIN,RXPNr     ;Kontrolle ob wirklich Startbit
        rjmp    SRCVC1
        ;Zeitschleife ganze Bitzeit; 2 Taktzyklen (sbic) kompensieren
        ldi     XL,LOW(((((10*Clock)/(4*Baud))-((3*10)/4)+5)/10))
        ldi     XH,HIGH(((((10*Clock)/(4*Baud))-((3*10)/4)+5)/10))
SRCVC3: sbiw    XL,1
        brne    SRCVC3
SRCVC4: clc                     ;Carry auf Null
        sbic    RXPIN,RXPNr     ;Bit abtasten
        sec                     ;Carry auf Eins
        ror     r24             ;Schiebe Carry nach rechts
        ;Zeitschleife ganze Bitzeit; 6 Taktzyklen (sbic+ror+dec+brne) komp.
        ldi     XL,LOW(((((10*Clock)/(4*Baud))-((7*10)/4)+5)/10))
        ldi     XH,HIGH(((((10*Clock)/(4*Baud))-((7*10)/4)+5)/10))
SRCVC5: sbiw    XL,1
        brne    SRCVC5

        dec     r25
        brne    SRCVC4

        pop     XH
        pop     XL
        ret
