;*******************************************************************************
;*
;*      Titel:  Soft-UART-Bibliothek: Unterprogramme zur Emulation einer
;*              seriellen Schnittstelle (EIA-232)
;*              (SR_UART_SOFT.asm)
;*
;*      Datum:  15/01/11        Version:        1.0
;*      Autor:  WEIGU
;*
;*      Informationen zur Beschaltung:
;*
;*      Prozessor: ATmega32A       Quarzfrequenz: 16MHz
;*      Eingaenge: Festlegung per Zuweisung
;*      Ausgaenge: Festlegung per Zuweisung
;*
;*      Unterprogramme:
;*      SSNDC:     Sende 1 Byte (r24) ueber die serielle Schnittstelle (8N1)
;*      SSNDC11:   Sende 1 Byte (r24) ueber die serielle Schnittstelle
;*                 (115200 Baud 8N1) Quarz 16MHz!! Fehler unter 0,5%
;*      SRECC:     Empfange 1 Byte (r24) ueber die serielle Schnittstelle (8N1)
;*      ISRSRC:    Empfange 1 Byte (r24) ueber die serielle Schnittstelle (8N1)
;*                 ISR nutzt externen Interrupt an PB2 (INT2)                 
;*                 
;*******************************************************************************
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Zuweisungen 
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.EQU    RXDDR   = DDRC
.EQU    RXPIN   = PINC
.EQU    RXPORT  = PORTC
.EQU    RXPNr   = 0

.EQU    TXDDR   = DDRC
.EQU    TXPORT  = PORTC
.EQU    TXPNr   = 1

.EQU    Baud    = 115200
.EQU    Takt    = 16000000

;-------------------------------------------------------------------------------
;       Sende 1 Byte (r24) ueber die serielle Schnittstelle (8N1)
;-------------------------------------------------------------------------------
SSNDC:  push    XL              ;rette verwendete Register
        push    XH

        sbi     TXDDR,TXPNr     ;Ausgang
        sbi     TXPORT,TXPNr    ;Leitung auf 1
        ldi     r25,8           ;Zaehler = 8

        cbi     TXPORT,TXPNr    ;Startbit senden
        ;Zeitschleife fuer das Startbit
        ldi     XL,LOW(((((10*Takt)/(4*Baud))-((2*10)/4)+5)/10))
        ldi     XH,HIGH(((((10*Takt)/(4*Baud))-((2*10)/4)+5)/10))
SSNDC1: sbiw    XL,1
        brne    SSNDC1
        ;Sende Zeichen LSB first!
SSNDC2: lsr     r24             ;Bit in den Uebertrag (Carry) schieben
        brcs    SSNDC3          ;Carry-Bit testen
        cbi     TXPORT,TXPNr    ;Null
        rjmp    SSNDC4          ;zur Zeitschleife
SSNDC3: sbi     TXPORT,TXPNr    ;Eins
        nop                     ;Zeitausgleich
SSNDC4: ;Zeitschleife fuer ein Datenbit
        ldi     XL,LOW(((((10*Takt)/(4*Baud))-((9*10)/4)+5)/10))
        ldi     XH,HIGH(((((10*Takt)/(4*Baud))-((9*10)/4)+5)/10))
SSNDC5: sbiw    XL,1
        brne    SSNDC5
        dec     r25             ;falls nicht alle Bits durch, dann naechstes Bit
        brne    SSNDC2

        nop                     ;Zeitausgleich für Sprungbefehl (MSB)
        sbi     TXPORT,TXPNr    ;Stoppbit senden
        ;Zeitschleife fuer das Stoppbit
        ldi     XL,LOW(((((10*Takt)/(4*Baud))-((2*10)/4)+5)/10))
        ldi     XH,HIGH(((((10*Takt)/(4*Baud))-((2*10)/4)+5)/10))
SSNDC6: sbiw    XL,1
        brne    SSNDC6

        pop     XH              ;Register wiederherstellen
        pop     XL
        ret                     ;zurueck

