;*******************************************************************************
;*
;*	Titel:	Unterprogrammbibliothek Watchdog (SR_WATCHDOG.asm)
;*	Datum:	15/04/12	Version:	0.2 (03/09/13)
;*	Autor:	WEIGU
;*
;*	Informationen zur Beschaltung:
;*      
;*      Prozessor:      ATmega644p	Quarzfrequenz:  
;*	Eingaenge:
;*	Ausgaenge:
;*
;*	Informationen:
;* 
;*      Copyright (c) 2012   Guy WEILER         weigu[at]weigu[dot]lu
;*
;*      Hiermit wird unentgeltlich, jeder Person, die eine Kopie der Software
;*      und der zugehoerigen Dokumentationen (die "Software") erhaelt, die
;*      Erlaubnis erteilt, uneingeschraenkt zu benutzen, inklusive und ohne
;*      Ausnahme, dem Recht, sie zu verwenden, kopieren, aendern, fusionieren,
;*      verlegen, verbreiten, unterlizenzieren und/oder zu verkaufen, und
;*      Personen, die diese Software erhalten, diese Rechte zu geben, unter den
;*      folgenden Bedingungen:
;*
;*      ;Der obige Urheberrechtsvermerk und dieser Erlaubnisvermerk sind in alle
;*      ;Kopien oder Teilkopien der Software beizulegen.
;*      ;
;*      ;DIE SOFTWARE WIRD OHNE JEDE AUSDRUECKLICHE ODER IMPLIZIERTE GARANTIE
;*      ;BEREITGESTELLT, EINSCHLIESSLICH DER GARANTIE ZUR BENUTZUNG FUER DEN
;*      ;VORGESEHENEN ODER EINEM BESTIMMTEN ZWECK SOWIE JEGLICHER RECHTS-
;*      ;VERLETZUNG, JEDOCH NICHT DARAUF BESCHRAENKT. IN KEINEM FALL SIND DIE
;*      ;AUTOREN ODER COPYRIGHTINHABER FUER JEGLICHEN SCHADEN ODER SONSTIGE
;*      ;ANSPRUECHE HAFTBAR ZU MACHEN, OB INFOLGE DER ERFUELLUNG EINES
;*      ;VERTRAGES, EINES DELIKTES ODER ANDERS IM ZUSAMMENHANG MIT DER SOFTWARE
;*      ;ODER SONSTIGER VERWENDUNG DER SOFTWARE ENTSTANDEN.
;*			
;*******************************************************************************
;///////////////////////////////////////////////////////////////////////////////
;	Makros 
;	siehe http://www.mikrocontroller.net/articles/AVR_Assembler_Makros
;///////////////////////////////////////////////////////////////////////////////
;Bei neuen Chips liegen viele SF-Register ueber den 64 Adressen die mit den 
;Befehlen "in" und "out" zu erreichen sind. Fuer sie muessen "sts" und "lds"
;verwendet werden. Dies wird mit den folgenden beiden Makros erledigt. Statt der
;Befehle "in" und "out" werden in der Bibliothek die Makros "input" und "output"
;verwendet.

;Einlesen eines Ports mit input Arbeitsregister,SF-Register
.IFNDEF MACROINPUT
.SET	MACROINPUT = 1
  .MACRO input
    .IF @1 < 64
	in	@0, @1
    .ELSE
	lds	@0, @1
    .ENDIF
  .ENDM
.ENDIF

;Schreiben eines Ports mit output SF-Register,Arbeitsregister
.IFNDEF MACROOUTPUT
.SET	MACROOUTPUT = 1
  .MACRO output
    .IF @0 < 64
	out	@0, @1
    .ELSE
  	sts	@0, @1
    .ENDIF
  .ENDM
.ENDIF

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Reservierungen und Zuweisungen
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;bei 5V
;0 16 ms
;1 32 ms
;2 64 ms
;3 0.125 s
;4 0.25 s
;5 0.5 s
;6 1 s
;7 2 s
;32 4 s
;33 8 s

.EQU	WDTTIME = 33


;ATmega32U4 
.IF ((SIGNATURE_000 == 0x1E) && (SIGNATURE_001 == 0x95) && (SIGNATURE_002 == 0x87))
.EQU    WDTCR   = WDTCSR
.EQU    MCUCSR  = MCUSR
.ENDIF

;ATmega644P  
.IF ((SIGNATURE_000 == 0x1E) && (SIGNATURE_001 == 0x96) && (SIGNATURE_002 == 0x0A))
.EQU    WDTCR   = WDTCSR
.EQU    MCUCSR  = MCUSR
.ENDIF
	
;-------------------------------------------------------------------------------
;	Watchdogtimer einschalten (Datenblatt)
;-------------------------------------------------------------------------------
WDT_ON:	push	r16
	in	r16,SREG
	push	r16

	cli
	wdr			;WDT Reset
	input	r16,WDTCR
	ori	r16,0x18	;WDCE und WDE einschalten
	output	WDTCR,r16
	ldi	r16,0x08	;WDE = 1, WDCE = 0!
	ori	r16,WDTTIME
	output	WDTCR,r16		
	sei
	
	pop	r16
	out	SREG,r16
	pop	r16
	ret

;-------------------------------------------------------------------------------
;       Watchdogtimer ausschalten (Datenblatt)
;-------------------------------------------------------------------------------
WDT_OF:	push	r16
	in	r16,SREG
	push	r16

	cli
	wdr			;WDT Reset
	input	r16,MCUCSR
	andi	r16,0xF7	;loesche Flag (WDRF) mit 0 
	output	MCUCSR,r16
	input	r16,WDTCR
	ori	r16,0x18	;WDCE und WDE einschalten
	output	WDTCR,r16
	output	WDTCR,Zero	;WDE = 0, WDCE = 0! WDT ausschalten	
	sei
	
	pop	r16
	out	SREG,r16
	pop	r16
	ret		